/*******************************************************************************
 * Copyright (c) 2010, 2014 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sonatype, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.aether.repository;

import org.eclipse.aether.RepositoryException;

/**
 * Thrown in case of an unsupported local repository type.
 */
public class NoLocalRepositoryManagerException
    extends RepositoryException
{

    private final transient LocalRepository repository;

    /**
     * Creates a new exception with the specified repository.
     * 
     * @param repository The local repository for which no support is available, may be {@code null}.
     */
    public NoLocalRepositoryManagerException( LocalRepository repository )
    {
        this( repository, toMessage( repository ) );
    }

    /**
     * Creates a new exception with the specified repository and detail message.
     * 
     * @param repository The local repository for which no support is available, may be {@code null}.
     * @param message The detail message, may be {@code null}.
     */
    public NoLocalRepositoryManagerException( LocalRepository repository, String message )
    {
        super( message );
        this.repository = repository;
    }

    /**
     * Creates a new exception with the specified repository and cause.
     * 
     * @param repository The local repository for which no support is available, may be {@code null}.
     * @param cause The exception that caused this one, may be {@code null}.
     */
    public NoLocalRepositoryManagerException( LocalRepository repository, Throwable cause )
    {
        this( repository, toMessage( repository ), cause );
    }

    /**
     * Creates a new exception with the specified repository, detail message and cause.
     * 
     * @param repository The local repository for which no support is available, may be {@code null}.
     * @param message The detail message, may be {@code null}.
     * @param cause The exception that caused this one, may be {@code null}.
     */
    public NoLocalRepositoryManagerException( LocalRepository repository, String message, Throwable cause )
    {
        super( message, cause );
        this.repository = repository;
    }

    private static String toMessage( LocalRepository repository )
    {
        if ( repository != null )
        {
            return "No manager available for local repository (" + repository.getBasedir().getAbsolutePath()
                + ") of type " + repository.getContentType();
        }
        else
        {
            return "No manager available for local repository";
        }
    }

    /**
     * Gets the local repository whose content type is not supported.
     * 
     * @return The unsupported local repository or {@code null} if unknown.
     */
    public LocalRepository getRepository()
    {
        return repository;
    }

}
